"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeProof = exports.serializeProof = exports.createNodeFromProof = exports.createProof = exports.ProofTypeSerialized = exports.ProofType = exports.descriptorToBitlist = exports.computeDescriptor = void 0;
const multi_js_1 = require("./multi.js");
const compactMulti_js_1 = require("./compactMulti.js");
const single_js_1 = require("./single.js");
const treeOffset_js_1 = require("./treeOffset.js");
var compactMulti_js_2 = require("./compactMulti.js");
Object.defineProperty(exports, "computeDescriptor", { enumerable: true, get: function () { return compactMulti_js_2.computeDescriptor; } });
Object.defineProperty(exports, "descriptorToBitlist", { enumerable: true, get: function () { return compactMulti_js_2.descriptorToBitlist; } });
var ProofType;
(function (ProofType) {
    ProofType["single"] = "single";
    ProofType["treeOffset"] = "treeOffset";
    ProofType["multi"] = "multi";
    ProofType["compactMulti"] = "compactMulti";
})(ProofType || (exports.ProofType = ProofType = {}));
/**
 * Serialized proofs are prepended with a single byte, denoting their type
 */
exports.ProofTypeSerialized = [
    ProofType.single, // 0
    ProofType.treeOffset, // 1
    ProofType.multi, // 2
    ProofType.compactMulti, // 3
];
function createProof(rootNode, input) {
    switch (input.type) {
        case ProofType.single: {
            const [leaf, witnesses] = (0, single_js_1.createSingleProof)(rootNode, input.gindex);
            return {
                type: ProofType.single,
                gindex: input.gindex,
                leaf,
                witnesses,
            };
        }
        case ProofType.treeOffset: {
            const [offsets, leaves] = (0, treeOffset_js_1.createTreeOffsetProof)(rootNode, input.gindices);
            return {
                type: ProofType.treeOffset,
                offsets,
                leaves,
            };
        }
        case ProofType.multi: {
            const [leaves, witnesses, gindices] = (0, multi_js_1.createMultiProof)(rootNode, input.gindices);
            return {
                type: ProofType.multi,
                leaves,
                witnesses,
                gindices,
            };
        }
        case ProofType.compactMulti: {
            const leaves = (0, compactMulti_js_1.createCompactMultiProof)(rootNode, input.descriptor);
            return {
                type: ProofType.compactMulti,
                leaves,
                descriptor: input.descriptor,
            };
        }
        default:
            throw new Error("Invalid proof type");
    }
}
exports.createProof = createProof;
function createNodeFromProof(proof) {
    switch (proof.type) {
        case ProofType.single:
            return (0, single_js_1.createNodeFromSingleProof)(proof.gindex, proof.leaf, proof.witnesses);
        case ProofType.treeOffset:
            return (0, treeOffset_js_1.createNodeFromTreeOffsetProof)(proof.offsets, proof.leaves);
        case ProofType.multi:
            return (0, multi_js_1.createNodeFromMultiProof)(proof.leaves, proof.witnesses, proof.gindices);
        case ProofType.compactMulti:
            return (0, compactMulti_js_1.createNodeFromCompactMultiProof)(proof.leaves, proof.descriptor);
        default:
            throw new Error("Invalid proof type");
    }
}
exports.createNodeFromProof = createNodeFromProof;
function serializeProof(proof) {
    switch (proof.type) {
        case ProofType.single:
        case ProofType.multi:
            throw new Error("Not implemented");
        case ProofType.treeOffset: {
            const output = new Uint8Array(1 + (0, treeOffset_js_1.computeTreeOffsetProofSerializedLength)(proof.offsets, proof.leaves));
            output[0] = exports.ProofTypeSerialized.indexOf(ProofType.treeOffset);
            (0, treeOffset_js_1.serializeTreeOffsetProof)(output, 1, proof.offsets, proof.leaves);
            return output;
        }
        default:
            throw new Error("Invalid proof type");
    }
}
exports.serializeProof = serializeProof;
function deserializeProof(data) {
    const proofType = exports.ProofTypeSerialized[data[0]];
    if (!proofType) {
        throw new Error("Invalid proof type");
    }
    switch (proofType) {
        case ProofType.single:
        case ProofType.multi:
            throw new Error("Not implemented");
        case ProofType.treeOffset: {
            const [offsets, leaves] = (0, treeOffset_js_1.deserializeTreeOffsetProof)(data, 1);
            return {
                type: ProofType.treeOffset,
                offsets,
                leaves,
            };
        }
        default:
            throw new Error("Invalid proof type");
    }
}
exports.deserializeProof = deserializeProof;
//# sourceMappingURL=index.js.map